@@ -195,7 +195,7 @@ gth_browser_activate_action_catalog_new (GtkAction  *action,
 	if (name == NULL)
 		return;
 
-	selected_parent = gth_folder_tree_get_selected_or_parent (GTH_FOLDER_TREE (gth_browser_get_folder_tree (browser)));
+	selected_parent = gth_browser_get_folder_popup_file_data (browser);
 	if (selected_parent != NULL) {
 		GthFileSource *file_source;
 		GFileInfo     *info;
@@ -261,7 +261,7 @@ gth_browser_activate_action_catalog_new_library (GtkAction  *action,
 	if (name == NULL)
 		return;
 
-	selected_parent = gth_folder_tree_get_selected_or_parent (GTH_FOLDER_TREE (gth_browser_get_folder_tree (browser)));
+	selected_parent = gth_browser_get_folder_popup_file_data (browser);
 	if (selected_parent != NULL) {
 		GthFileSource *file_source;
 		GFileInfo     *info;
@@ -305,17 +305,13 @@ gth_browser_activate_action_catalog_new_library (GtkAction  *action,
 }
 
 
-void
-gth_browser_activate_action_catalog_remove (GtkAction  *action,
-					    GthBrowser *browser)
+static void
+remove_catalog (GtkWindow   *window,
+		GthFileData *file_data)
 {
-	GthFolderTree *folder_tree;
-	GthFileData   *file_data;
-	GFile         *gio_file;
-	GError        *error = NULL;
+	GFile  *gio_file;
+	GError *error = NULL;
 
-	folder_tree = GTH_FOLDER_TREE (gth_browser_get_folder_tree (browser));
-	file_data = gth_folder_tree_get_selected (folder_tree);
 	gio_file = gth_main_get_gio_file (file_data->file);
 	if (g_file_delete (gio_file, NULL, &error)) {
 		GFile *parent;
@@ -332,25 +328,70 @@ gth_browser_activate_action_catalog_remove (GtkAction  *action,
 		_g_object_unref (parent);
 	}
 	else
-		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (browser),
+		_gtk_error_dialog_from_gerror_show (window,
 						    _("Could not remove the catalog"),
 						    &error);
 
 	g_object_unref (gio_file);
+}
+
+
+static void
+remove_catalog_response_cb (GtkDialog *dialog,
+			    int        response_id,
+			    gpointer   user_data)
+{
+	GthFileData *file_data = user_data;
+
+	if (response_id == GTK_RESPONSE_YES)
+		remove_catalog (gtk_window_get_transient_for (GTK_WINDOW (dialog)), file_data);
+
+	gtk_widget_destroy (GTK_WIDGET (dialog));
 	g_object_unref (file_data);
 }
 
 
+void
+gth_browser_activate_action_catalog_remove (GtkAction  *action,
+					    GthBrowser *browser)
+{
+	GthFileData *file_data;
+
+	file_data = gth_browser_get_folder_popup_file_data (browser);
+
+	if (eel_gconf_get_boolean (PREF_MSG_CONFIRM_DELETION, DEFAULT_MSG_CONFIRM_DELETION)) {
+		char      *prompt;
+		GtkWidget *d;
+
+		prompt = g_strdup_printf (_("Are you sure you want to remove \"%s\"?"), g_file_info_get_display_name (file_data->info));
+		d = _gtk_message_dialog_new (GTK_WINDOW (browser),
+					     GTK_DIALOG_MODAL,
+					     GTK_STOCK_DIALOG_QUESTION,
+					     prompt,
+					     NULL,
+					     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					     GTK_STOCK_REMOVE, GTK_RESPONSE_YES,
+					     NULL);
+		g_signal_connect (d, "response", G_CALLBACK (remove_catalog_response_cb), file_data);
+		gtk_widget_show (d);
+
+		g_free (prompt);
+	}
+	else {
+		remove_catalog (GTK_WINDOW (browser), file_data);
+		g_object_unref (file_data);
+	}
+}
+
+
 void
 gth_browser_activate_action_catalog_rename (GtkAction  *action,
 					    GthBrowser *browser)
 {
-	GthFolderTree *folder_tree;
-	GthFileData   *file_data;
+	GthFileData *file_data;
 
-	folder_tree = GTH_FOLDER_TREE (gth_browser_get_folder_tree (browser));
-	file_data = gth_folder_tree_get_selected (folder_tree);
-	gth_folder_tree_start_editing (folder_tree, file_data->file);
+	file_data = gth_browser_get_folder_popup_file_data (browser);
+	gth_folder_tree_start_editing (GTH_FOLDER_TREE (gth_browser_get_folder_tree (browser)), file_data->file);
 
 	g_object_unref (file_data);
 }
@@ -360,11 +401,9 @@ void
 gth_browser_activate_action_catalog_properties (GtkAction  *action,
 						GthBrowser *browser)
 {
-	GthFolderTree *folder_tree;
-	GthFileData   *file_data;
+	GthFileData *file_data;
 
-	folder_tree = GTH_FOLDER_TREE (gth_browser_get_folder_tree (browser));
-	file_data = gth_folder_tree_get_selected (folder_tree);
+	file_data = gth_browser_get_folder_popup_file_data (browser);
 	dlg_catalog_properties (browser, file_data);
 
 	g_object_unref (file_data);
