@@ -33,6 +33,7 @@ static gpointer gth_edit_metadata_dialog_parent_class = NULL;
 
 struct _GthEditMetadataDialogPrivate {
 	GtkWidget *notebook;
+	GtkWidget *save_changed_checkbutton;
 };
 
 
@@ -82,8 +83,9 @@ gth_edit_metadata_dialog_get_type (void)
 static void
 gth_edit_metadata_dialog_construct (GthEditMetadataDialog *self)
 {
-	GArray *pages;
-	int     i;
+	GtkWidget *vbox;
+	GArray    *pages;
+	int        i;
 
 	gtk_window_set_resizable (GTK_WINDOW (self), TRUE);
 	gtk_dialog_set_has_separator (GTK_DIALOG (self), FALSE);
@@ -93,10 +95,18 @@ gth_edit_metadata_dialog_construct (GthEditMetadataDialog *self)
 	gtk_dialog_add_button (GTK_DIALOG (self), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
 	gtk_dialog_add_button (GTK_DIALOG (self), GTK_STOCK_SAVE, GTK_RESPONSE_OK);
 
+	vbox = gtk_vbox_new (FALSE, 6);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
+	gtk_widget_show (vbox);
+	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (self))), vbox, TRUE, TRUE, 0);
+
 	self->priv->notebook = gtk_notebook_new ();
 	gtk_widget_show (self->priv->notebook);
-	gtk_container_set_border_width (GTK_CONTAINER (self->priv->notebook), 5);
-	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (self))), self->priv->notebook, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), self->priv->notebook, TRUE, TRUE, 0);
+
+	self->priv->save_changed_checkbutton = gtk_check_button_new_with_mnemonic (_("Save only cha_nged fields"));
+	gtk_widget_show (self->priv->save_changed_checkbutton);
+	gtk_box_pack_start (GTK_BOX (vbox), self->priv->save_changed_checkbutton, FALSE, FALSE, 0);
 
 	pages = gth_main_get_type_set ("edit-metadata-dialog-page");
 	if (pages == NULL)
@@ -134,23 +144,36 @@ gth_edit_metadata_dialog_new (void)
 
 
 void
-gth_edit_metadata_dialog_set_file (GthEditMetadataDialog *dialog,
-				   GthFileData           *file)
+gth_edit_metadata_dialog_set_file_list (GthEditMetadataDialog *dialog,
+				        GList                 *file_list)
 {
+	int    n_files;
 	char  *title;
 	GList *pages;
 	GList *scan;
 
-	if (file == NULL)
+	if (file_list == NULL)
 		return;
 
-	/* Translators: the %s in the string is the file name */
-	title = g_strdup_printf (_("%s Metadata"), g_file_info_get_display_name (file->info));
-	gtk_window_set_title (GTK_WINDOW (dialog), title);
+	n_files = g_list_length (file_list);
+	if (n_files == 1) {
+		GthFileData *file_data = file_list->data;
+
+		/* Translators: the %s symbol in the string is a file name */
+		title = g_strdup_printf (_("%s Metadata"), g_file_info_get_display_name (file_data->info));
+		gtk_window_set_title (GTK_WINDOW (dialog), title);
+	}
+	else {
+		title = g_strdup_printf (g_dngettext (NULL, "%d file", "%d files", n_files), n_files);
+		gtk_window_set_title (GTK_WINDOW (dialog), title);
+	}
+
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->priv->save_changed_checkbutton), n_files > 1);
+	gtk_widget_set_sensitive (dialog->priv->save_changed_checkbutton, n_files > 1);
 
 	pages = gtk_container_get_children (GTK_CONTAINER (dialog->priv->notebook));
 	for (scan = pages; scan; scan = scan->next)
-		gth_edit_metadata_page_set_file (GTH_EDIT_METADATA_PAGE (scan->data), file);
+		gth_edit_metadata_page_set_file_list (GTH_EDIT_METADATA_PAGE (scan->data), file_list);
 
 	g_list_free (pages);
 	g_free (title);
@@ -159,14 +182,22 @@ gth_edit_metadata_dialog_set_file (GthEditMetadataDialog *dialog,
 
 void
 gth_edit_metadata_dialog_update_info (GthEditMetadataDialog *dialog,
-				      GFileInfo             *info)
+				      GList                 *file_list /* GthFileData list */)
 {
-	GList *pages;
-	GList *scan;
+	gboolean  only_modified_fields;
+	GList    *pages;
+	GList    *scan;
 
+	only_modified_fields = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->priv->save_changed_checkbutton));
 	pages = gtk_container_get_children (GTK_CONTAINER (dialog->priv->notebook));
-	for (scan = pages; scan; scan = scan->next)
-		gth_edit_metadata_page_update_info (GTH_EDIT_METADATA_PAGE (scan->data), info);
+	for (scan = pages; scan; scan = scan->next) {
+		GList *scan_file;
+
+		for (scan_file = file_list; scan_file; scan_file = scan_file->next) {
+			GthFileData *file_data = scan_file->data;
+			gth_edit_metadata_page_update_info (GTH_EDIT_METADATA_PAGE (scan->data), file_data->info, only_modified_fields);
+		}
+	}
 
 	g_list_free (pages);
 }
@@ -198,18 +229,19 @@ gth_edit_metadata_page_get_type (void) {
 
 
 void
-gth_edit_metadata_page_set_file (GthEditMetadataPage *self,
-				 GthFileData         *file_data)
+gth_edit_metadata_page_set_file_list (GthEditMetadataPage *self,
+				      GList               *file_list)
 {
-	GTH_EDIT_METADATA_PAGE_GET_INTERFACE (self)->set_file (self, file_data);
+	GTH_EDIT_METADATA_PAGE_GET_INTERFACE (self)->set_file_list (self, file_list);
 }
 
 
 void
 gth_edit_metadata_page_update_info (GthEditMetadataPage *self,
-				    GFileInfo           *info)
+				    GFileInfo           *info,
+				    gboolean             only_modified_fields)
 {
-	GTH_EDIT_METADATA_PAGE_GET_INTERFACE (self)->update_info (self, info);
+	GTH_EDIT_METADATA_PAGE_GET_INTERFACE (self)->update_info (self, info, only_modified_fields);
 }
 
 
