@@ -199,20 +199,20 @@ static GtkActionEntry image_viewer_action_entries[] = {
 
 
 static void
-image_ready_cb (GtkWidget          *widget,
-		GthImageViewerPage *self)
+viewer_image_ready_cb (GtkWidget          *widget,
+		       GthImageViewerPage *self)
 {
 	gth_image_history_clear (self->priv->history);
 	gth_image_history_add_image (self->priv->history,
 				     gth_image_viewer_get_current_pixbuf (GTH_IMAGE_VIEWER (self->priv->viewer)),
 				     FALSE);
-	gth_viewer_page_file_loaded (GTH_VIEWER_PAGE (self));
+	gth_viewer_page_file_loaded (GTH_VIEWER_PAGE (self), TRUE);
 }
 
 
 static gboolean
-zoom_changed_cb (GtkWidget          *widget,
-		 GthImageViewerPage *self)
+viewer_zoom_changed_cb (GtkWidget          *widget,
+			GthImageViewerPage *self)
 {
 	double  zoom;
 	char   *text;
@@ -230,23 +230,33 @@ zoom_changed_cb (GtkWidget          *widget,
 
 
 static gboolean
-image_button_press_cb (GtkWidget          *widget,
-		       GdkEventButton     *event,
-		       GthImageViewerPage *self)
+viewer_button_press_event_cb (GtkWidget          *widget,
+			      GdkEventButton     *event,
+			      GthImageViewerPage *self)
 {
 	return gth_browser_viewer_button_press_cb (self->priv->browser, event);
 }
 
 
 static gboolean
-mouse_whell_scroll_cb (GtkWidget 	   *widget,
-		       GdkEventScroll      *event,
-		       GthImageViewerPage  *self)
+viewer_scroll_event_cb (GtkWidget 	   *widget,
+		        GdkEventScroll      *event,
+		        GthImageViewerPage  *self)
 {
 	return gth_browser_viewer_scroll_event_cb (self->priv->browser, event);
 }
 
 
+static gboolean
+viewer_image_map_event_cb (GtkWidget          *widget,
+			   GdkEvent           *event,
+			   GthImageViewerPage *self)
+{
+	gth_viewer_page_focus (GTH_VIEWER_PAGE (self));
+	return FALSE;
+}
+
+
 static gboolean
 viewer_key_press_cb (GtkWidget          *widget,
 		     GdkEventKey        *event,
@@ -264,16 +274,17 @@ image_preloader_requested_ready_cb (GthImagePreloader  *preloader,
 	GthImageLoader *image_loader;
 
 	if (error != NULL) {
-		gth_image_viewer_set_void (GTH_IMAGE_VIEWER (self->priv->viewer));
+		gth_viewer_page_file_loaded (GTH_VIEWER_PAGE (self), FALSE);
 		return;
 	}
 
 	image_loader = gth_image_preloader_get_loader (self->priv->preloader, gth_image_preloader_get_requested (self->priv->preloader));
-	if (image_loader == NULL)
+	if (image_loader == NULL) {
+		gth_viewer_page_file_loaded (GTH_VIEWER_PAGE (self), FALSE);
 		return;
+	}
 
-	if (GTK_WIDGET_VISIBLE (self->priv->viewer))
-		gth_viewer_page_focus (GTH_VIEWER_PAGE (self));
+	gth_viewer_page_focus (GTH_VIEWER_PAGE (self));
 
 	gth_image_viewer_load_from_image_loader (GTH_IMAGE_VIEWER (self->priv->viewer), image_loader);
 }
@@ -413,19 +424,23 @@ gth_image_viewer_page_real_activate (GthViewerPage *base,
 
 	g_signal_connect (G_OBJECT (self->priv->viewer),
 			  "image_ready",
-			  G_CALLBACK (image_ready_cb),
+			  G_CALLBACK (viewer_image_ready_cb),
 			  self);
 	g_signal_connect (G_OBJECT (self->priv->viewer),
 			  "zoom_changed",
-			  G_CALLBACK (zoom_changed_cb),
+			  G_CALLBACK (viewer_zoom_changed_cb),
 			  self);
 	g_signal_connect_after (G_OBJECT (self->priv->viewer),
 				"button_press_event",
-				G_CALLBACK (image_button_press_cb),
+				G_CALLBACK (viewer_button_press_event_cb),
 				self);
 	g_signal_connect_after (G_OBJECT (self->priv->viewer),
-				"mouse_wheel_scroll",
-				G_CALLBACK (mouse_whell_scroll_cb),
+				"scroll_event",
+				G_CALLBACK (viewer_scroll_event_cb),
+				self);
+	g_signal_connect_after (G_OBJECT (self->priv->viewer),
+				"map_event",
+				G_CALLBACK (viewer_image_map_event_cb),
 				self);
 	g_signal_connect (G_OBJECT (self->priv->viewer),
 			  "key_press_event",
@@ -606,7 +621,7 @@ gth_image_viewer_page_real_focus (GthViewerPage *base)
 	GtkWidget *widget;
 
 	widget = GTH_IMAGE_VIEWER_PAGE (base)->priv->viewer;
-	if (GTK_WIDGET_REALIZED (widget))
+	if (GTK_WIDGET_REALIZED (widget) && GTK_WIDGET_MAPPED (widget))
 		gtk_widget_grab_focus (widget);
 }
 
@@ -763,6 +778,7 @@ _gth_image_viewer_page_real_save (GthViewerPage *base,
 	_gdk_pixbuf_save_async (gth_image_viewer_get_current_pixbuf (GTH_IMAGE_VIEWER (self->priv->viewer)),
 			        current_file,
 			        mime_type,
+			        TRUE,
 				image_saved_cb,
 				data);
 }
@@ -878,7 +894,7 @@ gth_image_viewer_page_real_save_as (GthViewerPage *base,
 			  data);
 
 	gtk_window_set_transient_for (GTK_WINDOW (file_sel), GTK_WINDOW (self->priv->browser));
-	gtk_window_set_modal (GTK_WINDOW (file_sel), TRUE);
+	gtk_window_set_modal (GTK_WINDOW (file_sel), FALSE);
 	gtk_widget_show (file_sel);
 
 	g_free (uri);
@@ -906,7 +922,7 @@ _gth_image_viewer_page_set_pixbuf (GthImageViewerPage *self,
 	g_file_info_set_attribute_int32 (file_data->info, "image::width", width);
 	g_file_info_set_attribute_int32 (file_data->info, "image::height", height);
 
-	size = g_strdup_printf ("%d x %d", width, height);
+	size = g_strdup_printf ("%d × %d", width, height);
 	g_file_info_set_attribute_string (file_data->info, "general::dimensions", size);
 
 	gth_monitor_metadata_changed (gth_main_get_default_monitor (), file_data);
