@@ -66,7 +66,6 @@ enum {
 struct _GthImageViewerPrivate {
 	gboolean                is_animation;
 	gboolean                play_animation;
-	gboolean                rendering;
 	gboolean                cursor_visible;
 
 	gboolean                frame_visible;
@@ -496,6 +495,9 @@ gth_image_viewer_size_allocate (GtkWidget       *widget,
 	gdk_width = allocation->width - viewer->priv->frame_border2;
 	gdk_height = allocation->height - viewer->priv->frame_border2;
 
+	if ((gdk_width < 0) || (gdk_height < 0))
+		return;
+
 	current_pixbuf = gth_image_viewer_get_current_pixbuf (viewer);
 
 	/* If a fit type is active update the zoom level. */
@@ -602,8 +604,10 @@ gth_image_viewer_size_allocate (GtkWidget       *widget,
 
 	if (GTK_WIDGET_REALIZED (widget))
 		gdk_window_move_resize (widget->window,
-					allocation->x, allocation->y,
-					allocation->width, allocation->height);
+					allocation->x,
+					allocation->y,
+					allocation->width,
+					allocation->height);
 
 	gth_image_viewer_tool_size_allocate (viewer->priv->tool, allocation);
 
@@ -718,11 +722,6 @@ gth_image_viewer_expose (GtkWidget      *widget,
 
 	viewer = GTH_IMAGE_VIEWER (widget);
 
-	if (viewer->priv->rendering)
-		return FALSE;
-
-	viewer->priv->rendering = TRUE;
-
 	/* Draw the background. */
 
 	gdk_width = widget->allocation.width - viewer->priv->frame_border2;
@@ -861,8 +860,6 @@ gth_image_viewer_expose (GtkWidget      *widget,
 
 	gth_image_viewer_tool_expose (viewer->priv->tool, &event->area);
 
-	viewer->priv->rendering = FALSE;
-
 	/* Draw the focus. */
 
 #if 0
@@ -988,9 +985,6 @@ scroll_to (GthImageViewer *viewer,
 	if (gth_image_viewer_get_current_pixbuf (viewer) == NULL)
 		return;
 
-	if (viewer->priv->rendering)
-		return;
-
 	get_zoomed_size (viewer, &width, &height, viewer->priv->zoom_level);
 
 	drawable = GTK_WIDGET (viewer)->window;
@@ -1086,29 +1080,6 @@ scroll_to (GthImageViewer *viewer,
 		     viewer->priv->frame_border,
 		     abs (delta_x),
 		     gdk_height);
-
-	/* Process graphics exposures */
-
-#if 0 /* FIXME: gdk_event_get_graphics_expose is deprecated now */
-	replay_animation = viewer->priv->play_animation;
-	viewer->priv->play_animation = FALSE;
-	while ((event = gdk_event_get_graphics_expose (drawable)) != NULL) {
-		GdkEventExpose *expose = (GdkEventExpose*) event;
-
-		expose_area (viewer,
-			     expose->area.x,
-			     expose->area.y,
-			     expose->area.width,
-			     expose->area.height);
-
-		if (expose->count == 0) {
-			gdk_event_free (event);
-			break;
-		}
-		gdk_event_free (event);
-	}
-	viewer->priv->play_animation = replay_animation;
-#endif
 }
 
 
@@ -1118,9 +1089,6 @@ gth_image_viewer_motion_notify (GtkWidget      *widget,
 {
 	GthImageViewer *viewer = GTH_IMAGE_VIEWER (widget);
 
-	if (viewer->priv->rendering)
-		return FALSE;
-
 	if (viewer->pressed) {
 		viewer->drag_x = event->x + viewer->x_offset;
 		viewer->drag_y = event->y + viewer->y_offset;
@@ -1670,7 +1638,6 @@ image_loader_ready_cb (GthImageLoader *il,
 
 	if (error != NULL) {
 		g_clear_error (&error);
-
 		gth_image_viewer_set_void (viewer);
 		g_signal_emit (G_OBJECT (viewer),
 			       gth_image_viewer_signals[IMAGE_READY],
@@ -1740,7 +1707,6 @@ static void
 gth_image_viewer_instance_init (GthImageViewer *viewer)
 {
 	GTK_WIDGET_SET_FLAGS (viewer, GTK_CAN_FOCUS);
-	/*GTK_WIDGET_UNSET_FLAGS (viewer, GTK_DOUBLE_BUFFERED); FIXME: check if this is correct */
 
 	viewer->priv = GTH_IMAGE_VIEWER_GET_PRIVATE (viewer);
 
@@ -1753,7 +1719,6 @@ gth_image_viewer_instance_init (GthImageViewer *viewer)
 
 	viewer->priv->is_animation = FALSE;
 	viewer->priv->play_animation = TRUE;
-	viewer->priv->rendering = FALSE;
 	viewer->priv->cursor_visible = TRUE;
 
 	viewer->priv->frame_visible = TRUE;
@@ -2007,7 +1972,7 @@ gth_image_viewer_set_pixbuf (GthImageViewer *viewer,
 {
 	g_return_if_fail (viewer != NULL);
 
-	if (viewer->priv->is_animation || viewer->priv->rendering)
+	if (viewer->priv->is_animation)
 		return;
 
 	viewer->priv->is_void = (pixbuf == NULL);
@@ -2179,9 +2144,10 @@ gth_image_viewer_step_animation (GthImageViewer *viewer)
 {
 	g_return_if_fail (viewer != NULL);
 
-	if (!viewer->priv->is_animation) return;
-	if (viewer->priv->play_animation) return;
-	if (viewer->priv->rendering) return;
+	if (! viewer->priv->is_animation)
+		return;
+	if (viewer->priv->play_animation)
+		return;
 
 	change_frame_cb (viewer);
 }
@@ -2459,9 +2425,6 @@ gth_image_viewer_scroll_to (GthImageViewer *viewer,
 	if (gth_image_viewer_get_current_pixbuf (viewer) == NULL)
 		return;
 
-	if (viewer->priv->rendering)
-		return;
-
 	scroll_to (viewer, &x_offset, &y_offset);
 
 	g_signal_handlers_block_by_data (G_OBJECT (viewer->hadj), viewer);
@@ -2634,6 +2597,8 @@ gth_image_viewer_paint (GthImageViewer *viewer,
 	double         zoom_level;
 	int            bits_per_sample;
 	GdkColorspace  color_space;
+	guchar        *pixels;
+	int            rowstride;
 
 	zoom_level = viewer->priv->zoom_level;
 
@@ -2661,39 +2626,48 @@ gth_image_viewer_paint (GthImageViewer *viewer,
 		viewer->priv->paint_bps = bits_per_sample;
 	}
 
-	if (gdk_pixbuf_get_has_alpha (pixbuf))
-		gdk_pixbuf_composite_color (pixbuf,
-					    viewer->priv->paint_pixbuf,
-					    0, 0,
-					    width, height,
-					    (double) -src_x,
-					    (double) -src_y,
-					    zoom_level,
-					    zoom_level,
-					    interp_type,
-					    255,
-					    src_x, src_y,
-					    viewer->priv->check_size,
-					    viewer->priv->check_color1,
-					    viewer->priv->check_color2);
-	else
-		gdk_pixbuf_scale (pixbuf,
-				  viewer->priv->paint_pixbuf,
-				  0, 0,
-				  width, height,
-				  (double) -src_x,
-				  (double) -src_y,
-				  zoom_level,
-				  zoom_level,
-				  interp_type);
+	if ((zoom_level == 1.0) && ! gdk_pixbuf_get_has_alpha (pixbuf) && (bits_per_sample == 8)) {
+		rowstride = gdk_pixbuf_get_rowstride (pixbuf);
+		pixels = gdk_pixbuf_get_pixels (pixbuf) + (src_y * rowstride) + (src_x * gdk_pixbuf_get_n_channels (pixbuf));
+	}
+	else {
+		if (gdk_pixbuf_get_has_alpha (pixbuf))
+			gdk_pixbuf_composite_color (pixbuf,
+						    viewer->priv->paint_pixbuf,
+						    0, 0,
+						    width, height,
+						    (double) -src_x,
+						    (double) -src_y,
+						    zoom_level,
+						    zoom_level,
+						    interp_type,
+						    255,
+						    src_x, src_y,
+						    viewer->priv->check_size,
+						    viewer->priv->check_color1,
+						    viewer->priv->check_color2);
+		else
+			gdk_pixbuf_scale (pixbuf,
+					  viewer->priv->paint_pixbuf,
+					  0, 0,
+					  width, height,
+					  (double) -src_x,
+					  (double) -src_y,
+					  zoom_level,
+					  zoom_level,
+					  interp_type);
+
+		rowstride = gdk_pixbuf_get_rowstride (viewer->priv->paint_pixbuf);
+		pixels = gdk_pixbuf_get_pixels (viewer->priv->paint_pixbuf);
+	}
 
 	gdk_draw_rgb_image_dithalign (GTK_WIDGET (viewer)->window,
 				      GTK_WIDGET (viewer)->style->black_gc,
 				      dest_x, dest_y,
 				      width, height,
 				      GDK_RGB_DITHER_MAX,
-				      gdk_pixbuf_get_pixels (viewer->priv->paint_pixbuf),
-				      gdk_pixbuf_get_rowstride (viewer->priv->paint_pixbuf),
+				      pixels,
+				      rowstride,
 				      dest_x, dest_y);
 
 #if 0
